# Template Typescript Project

TODO : package manager, comment initialiser des projets (et ensuite juste copier la config eslint ou prettier, etc...)

npm, node, les autres commandes... Rmq : les deps eslint ?

[![Python Version](https://img.shields.io/badge/python-3.11-blue.svg)](https://www.python.org/downloads/release/python-311/)
[![Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
[![Ruff](https://img.shields.io/badge/linter-ruff-1a53f0.svg)](https://beta.ruff.rs/docs/)

## VSCode Project Settings

In order to ensure that everyone uses the same linting and formatting settings,
we duplicate the relevant settings in [`.vscode/settings.json`](.vscode/settings.json).

Relevant settings with python :

- format on save
- strict type checking
- format using **Black**

## Formatter - Black

[Black](https://github.com/psf/black) is a popular python formatter which is strict and opinionated.
Using a formatter ensures that the code you and your team write is consistent.
You should always use a formatter.

Additional configuration for the formatter can be added to the [`pyproject.toml`](./pyproject.toml) file,
although the default settings are already pretty good.

## Linter - Ruff

[Ruff](https://github.com/astral-sh/ruff) is a good python linter, in the sense that it is strict enough, and coded in rust (blazingly fast).
It is the fastest python linter available, which is nice when you want reactive linting during typing.
Using a linter will help you write cleaner code, and avoid common mistakes.
You should always use a linter.

Additional configuration for the formatter can be added to the [`pyproject.toml`](./pyproject.toml) file,
although the default settings are already pretty good.

## Environment and reproducibility

In order to make it easier for other developers to contribute to your project,
and to enable deploying your application easily, you must indicate which python version and dependencies you are using.

You typically indicate the python version in your [`pyproject.toml`](./pyproject.toml) or (`README.md`)(./README.md) file.

### Create a virtual environment

If you do not use `conda`, you may create a virtual environment. Virtual environments are often contained in a `venv` folder.
Virtual environment enable you to have separate python versions and dependencies for each project.

```bash
python --version # Ensure that your current python version is the one you want to use. Use `pyenv` to change it if needed (see wikibr).
python -m venv venv # Creates a virtual environment in the `venv` folder.

# LINUX / MACOS : activate the virtual environment
source venv/bin/activate

# WINDOWS : activate the virtual environment
venv\Scripts\activate
```

You must always activate your virtual environment before running your code or installing dependencies (else it will use your global python interpreter).

### Install and Freeze dependencies

Python depdendencies are typically indicated in a [`requirements.txt`](./requirements.txt) file.

Install dependencies :

```bash
pip install --upgrade pip # Update pip (python package manager)
pip install -r requirements.txt # Install dependencies read from the requirements file
```

Freeze dependencies :
You can write your dependencies by hand in the `requirements.txt` file, or you can "freeze" your current environment.

```bash
pip freeze > requirements.txt # Write the current dependencies to the requirements file
```

### Git

Some common gitignores for a python project :

- `__pycache__` : python cache, generated at runtime.
- `venv` : virtual environment folder.

## Python Clean Code

Some often-overlooked python clean code rules :

### Docstrings

Always write docstrings in order to describe the purpose of your functions and classes.

```python
def function():
    """This function does something."""
    pass
```

### Type hinting

When you can (always), use type hints to indicate the types of the parameters of your functions, their return types, etc...
You sometimes need to use the `typing` module to indicate more complex types.

```python
from typing import List
def add_to_list(my_list: List[int], element: int) -> List[int]:
    my_list.append(element)
    return my_list
```

Since python 3.10, you can use the `|` operator to indicate that a parameter can be of multiple types, and use `list` and `dict` directly as type hints.

```python
def add_to_list(my_list: list[int | float], element: int | float) -> list[int | float]:
    my_list.append(element)
    return my_list
```

### Use modules

When your code gets big, you should split it into multiple files.

Use the following test in order to run module-specific code only when you run your module directly via `python module.py`, and not when you import it from another file.

```python
def module_function():
    pass

if __name__ == "__main__":
    module_function()
```
